@@ -5,37 +5,68 @@
 
     Glue code for the jinja2 templating engine.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 from os import path
 from pprint import pformat
+from typing import Any, Callable, Iterator, Tuple  # NOQA
 
-from six import string_types
 from jinja2 import FileSystemLoader, BaseLoader, TemplateNotFound, \
     contextfunction
-from jinja2.utils import open_if_exists
 from jinja2.sandbox import SandboxedEnvironment
+from jinja2.utils import open_if_exists
+from six import string_types
 
 from sphinx.application import TemplateBridge
+from sphinx.util import logging
 from sphinx.util.osutil import mtimes_of_files
 
+if False:
+    # For type annotation
+    from typing import Any, Callable, Dict, List, Iterator, Tuple, Union  # NOQA
+    from jinja2.environment import Environment  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.theming import Theme  # NOQA
+
 
 def _tobool(val):
+    # type: (unicode) -> bool
     if isinstance(val, string_types):
         return val.lower() in ('true', '1', 'yes', 'on')
     return bool(val)
 
 
 def _toint(val):
+    # type: (unicode) -> int
     try:
         return int(val)
     except ValueError:
         return 0
 
 
+def _todim(val):
+    # type: (Union[int, unicode]) -> unicode
+    """
+    Make val a css dimension. In particular the following transformations
+    are performed:
+
+    - None -> 'initial' (default CSS value)
+    - 0 -> '0'
+    - ints and string representations of ints are interpreted as pixels.
+
+    Everything else is returned unchanged.
+    """
+    if val is None:
+        return 'initial'
+    elif str(val).isdigit():
+        return '0' if int(val) == 0 else '%spx' % val
+    return val  # type: ignore
+
+
 def _slice_index(values, slices):
+    # type: (List, int) -> Iterator[List]
     seq = list(values)
     length = 0
     for value in values:
@@ -57,6 +88,7 @@ def _slice_index(values, slices):
 
 
 def accesskey(context, key):
+    # type: (Any, unicode) -> unicode
     """Helper to output each access key only once."""
     if '_accesskeys' not in context:
         context.vars['_accesskeys'] = {}
@@ -68,17 +100,31 @@ def accesskey(context, key):
 
 class idgen(object):
     def __init__(self):
+        # type: () -> None
         self.id = 0
 
     def current(self):
+        # type: () -> int
         return self.id
 
     def __next__(self):
+        # type: () -> int
         self.id += 1
         return self.id
     next = __next__  # Python 2/Jinja compatibility
 
 
+@contextfunction
+def warning(context, message, *args, **kwargs):
+    # type: (Dict, unicode, Any, Any) -> unicode
+    if 'pagename' in context:
+        filename = context.get('pagename') + context.get('file_suffix', '')
+        message = 'in rendering %s: %s' % (filename, message)
+    logger = logging.getLogger('sphinx.themes')
+    logger.warning(message, *args, **kwargs)
+    return ''  # return empty string not to output any values
+
+
 class SphinxFileSystemLoader(FileSystemLoader):
     """
     FileSystemLoader subclass that is not so strict about '..'  entries in
@@ -86,6 +132,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
     """
 
     def get_source(self, environment, template):
+        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
         for searchpath in self.searchpath:
             filename = path.join(searchpath, template)
             f = open_if_exists(filename)
@@ -97,6 +144,7 @@ class SphinxFileSystemLoader(FileSystemLoader):
             mtime = path.getmtime(filename)
 
             def uptodate():
+                # type: () -> bool
                 try:
                     return path.getmtime(filename) == mtime
                 except OSError:
@@ -113,12 +161,13 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
     # TemplateBridge interface
 
     def init(self, builder, theme=None, dirs=None):
+        # type: (Builder, Theme, List[unicode]) -> None
         # create a chain of paths to search
         if theme:
             # the theme's own dir and its bases' dirs
-            pathchain = theme.get_dirchain()
-            # then the theme parent paths
-            loaderchain = pathchain + theme.themepath
+            pathchain = theme.get_theme_dirs()
+            # the loader dirs: pathchain + the parent directories for all themes
+            loaderchain = pathchain + [path.join(p, '..') for p in pathchain]
         elif dirs:
             pathchain = list(dirs)
             loaderchain = list(dirs)
@@ -146,26 +195,31 @@ class BuiltinTemplateLoader(TemplateBridge, BaseLoader):
                                                 extensions=extensions)
         self.environment.filters['tobool'] = _tobool
         self.environment.filters['toint'] = _toint
+        self.environment.filters['todim'] = _todim
         self.environment.filters['slice_index'] = _slice_index
         self.environment.globals['debug'] = contextfunction(pformat)
+        self.environment.globals['warning'] = warning
         self.environment.globals['accesskey'] = contextfunction(accesskey)
         self.environment.globals['idgen'] = idgen
         if use_i18n:
-            self.environment.install_gettext_translations(
-                builder.app.translator)
+            self.environment.install_gettext_translations(builder.app.translator)  # type: ignore  # NOQA
 
-    def render(self, template, context):
+    def render(self, template, context):  # type: ignore
+        # type: (unicode, Dict) -> unicode
         return self.environment.get_template(template).render(context)
 
     def render_string(self, source, context):
+        # type: (unicode, Dict) -> unicode
         return self.environment.from_string(source).render(context)
 
     def newest_template_mtime(self):
+        # type: () -> float
         return max(mtimes_of_files(self.pathchain, '.html'))
 
     # Loader interface
 
     def get_source(self, environment, template):
+        # type: (Environment, unicode) -> Tuple[unicode, unicode, Callable]
         loaders = self.loaders
         # exclamation mark starts search from theme
         if template.startswith('!'):
