@@ -5,7 +5,7 @@
 
     Create a full-text search index for offline search.
 
-    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 import re
@@ -85,7 +85,7 @@ var Stemmer = function() {
         at white spaces, which should be enough for most languages except CJK
         languages.
         """
-        return self._word_re.findall(input)  # type: ignore
+        return self._word_re.findall(input)
 
     def stem(self, word):
         # type: (unicode) -> unicode
